"use strict";
/**
Copyright (c) 2013 The Chromium Authors. All rights reserved.
Use of this source code is governed by a BSD-style license that can be
found in the LICENSE file.
**/

require("../base/guid.js");
require("../base/time_display_modes.js");
require("./event.js");

'use strict';

global.tr.exportTo('tr.model', function () {
  /**
   * TimedEvent is a base type for any entity in the trace model with a specific
   * start and duration.
   *
   * @constructor
   */
  function TimedEvent(start) {
    tr.model.Event.call(this);
    this.start = start;
    this.duration = 0;
    this.cpuStart = undefined;
    this.cpuDuration = undefined;
    // The set of contexts this event belongs to (order is unimportant). This
    // array should never be modified.
    this.contexts = Object.freeze([]);
  }

  TimedEvent.prototype = {
    __proto__: tr.model.Event.prototype,

    get end() {
      return this.start + this.duration;
    },

    addBoundsToRange: function (range) {
      range.addValue(this.start);
      range.addValue(this.end);
    },

    // TODO(charliea): Can this be implemented in terms of Event.range()?
    // Returns true if 'that' TimedEvent is fully contained within 'this' timed
    // event.
    bounds: function (that, opt_precisionUnit) {
      if (opt_precisionUnit === undefined) opt_precisionUnit = tr.b.TimeDisplayModes.ms;

      var startsBefore = opt_precisionUnit.roundedLess(that.start, this.start);
      var endsAfter = opt_precisionUnit.roundedLess(this.end, that.end);
      return !startsBefore && !endsAfter;
    }
  };

  return {
    TimedEvent: TimedEvent
  };
});